#include "mpi.h"
#include <stdio.h>
#include <math.h>

double f(double a)
{
    return (4.0 / (1.0 + a*a));
}

int main(int argc, char *argv[])
{
    int done = 0, n, myid, numprocs, i;
    double PI25DT = 3.141592653589793238462643;
    double mypi, pi, h, sum, x;
    double startwtime, endwtime;
    int  namelen;
    char processor_name[MPI_MAX_PROCESSOR_NAME];
	
    MPI_Init(&argc,&argv);
    MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
    MPI_Comm_rank(MPI_COMM_WORLD,&myid);
    MPI_Get_processor_name(processor_name,&namelen);
	
	if (myid == 0)
		startwtime = MPI_Wtime();

	fprintf(stderr,"Process %d on %s\n", myid, processor_name);
	fflush(stderr);
	
    n = 1000;
	h = 1.0 / (double) n;
    sum = 0.0;
    
	for (i = myid + 1; i <= n; i += numprocs)
    {
		x = h * ((double)i - 0.5);
		sum += f(x);
    }
    
	mypi = h * sum;
	MPI_Reduce(&mypi, &pi, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
			
    if (myid == 0)
	{
		FILE* f;
		f = fopen("pi.txt", "w");
		fprintf(f, "pi is approximately %.16f, Error is %.16f\n", pi, fabs(pi - PI25DT));
		endwtime = MPI_Wtime();
		fprintf(f, "wall clock time = %f\n", endwtime-startwtime);
		fclose(f);
		//printf("pi is approximately %.16f, Error is %.16f\n", pi, fabs(pi - PI25DT));
		//endwtime = MPI_Wtime();
		//printf("wall clock time = %f\n", endwtime-startwtime);
	}

    MPI_Finalize();
    return 0;
}
